--[[
    Armory Addon for World of Warcraft(tm).
    Revision: 87 2009-09-30T18:08:23Z
    URL: http://www.wow-neighbours.com

    License:
        This program is free software; you can redistribute it and/or
        modify it under the terms of the GNU General Public License
        as published by the Free Software Foundation; either version 2
        of the License, or (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program(see GPL.txt); if not, write to the Free Software
        Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    Note:
        This AddOn's source code is specifically designed to work with
        World of Warcraft's interpreted AddOn system.
        You have an implicit licence to use this AddOn with these facilities
        since that is it's designated purpose as per:
        http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
--]]

local Armory = Armory;
local container = "PawnScales";

function Armory:IsPawnLoaded()
    return PawnGetAllScales
       and PawnIsScaleVisible
       and PawnGetScaleTag
       and PawnResetScales
       and PawnDeleteScale
       and PawnImportScale
       and PawnRenameScale
       and PawnUIFrame_CurrentScaleDropDown_Reset
       and true;
end

function Armory:UpdatePawnScales()
    local dbEntry = self.playerDbBaseEntry;

    if ( not self:IsLocked(container) ) then
        self:Lock(container);

        self:PrintDebug("UPDATE", container);

        dbEntry:SetValue(container, nil);

        if ( self:IsPawnLoaded() and not self:GetConfigAltPawnScales("player") ) then
            local tags = {};
            for _, scale in ipairs(PawnGetAllScales()) do
                if ( PawnIsScaleVisible(scale) ) then
                    table.insert(tags, PawnGetScaleTag(scale));
                end
            end
            if ( #tags > 0 ) then
                dbEntry:SetValue(container, unpack(tags));
            end
        end

        self:Unlock(container);
    else
        self:PrintDebug("LOCKED", container);
    end
end

function Armory:LoadPawnScales()
    local currentProfile = self:CurrentProfile();
    local dbEntry;
    local scale, unitClass, unitColor;
    local pawnColor, tags;

    if ( self:IsPawnLoaded() and self:GetConfigAltPawnScales("player") ) then
        pawnColor = VgerCore.Color.Blue;
        PawnResetScales();
        PawnDeleteScale(PawnDefaultScaleName);

        for _, profile in ipairs(self:Profiles()) do
            if ( profile.realm == self.playerRealm ) then
                self:SelectProfile(profile);
                dbEntry = self.selectedDbBaseEntry;
                if ( dbEntry:Contains(container) and not self:GetConfigAltPawnScales() ) then
                    tags = { dbEntry:GetValue(container) };
                    for _, tag in ipairs(tags) do
                        _, scale = PawnImportScale(tag);
                        _, unitClass = self:UnitClass("player");
                        if ( unitClass ) then
                            unitColor = self:HexColor(GetTableColor(RAID_CLASS_COLORS[unitClass]));
                        else
                            unitColor = HIGHLIGHT_FONT_COLOR_CODE;
                        end
                        PawnRenameScale(scale, format("%s: %s", unitColor..profile.character..pawnColor, scale));
                    end
                end
            end
        end
        self:SelectProfile(currentProfile);

        PawnUIFrame_CurrentScaleDropDown_Reset();
    end
end

function Armory:ResetPawnScales()
    if ( self:IsPawnLoaded() ) then
        PawnResetScales();
    end
end