--[[
    Armory Addon for World of Warcraft(tm).
    Revision: 142 2009-11-04T12:40:43Z
    URL: http://www.wow-neighbours.com

    License:
        This program is free software; you can redistribute it and/or
        modify it under the terms of the GNU General Public License
        as published by the Free Software Foundation; either version 2
        of the License, or (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program(see GPL.txt); if not, write to the Free Software
        Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    Note:
        This AddOn's source code is specifically designed to work with
        World of Warcraft's interpreted AddOn system.
        You have an implicit licence to use this AddOn with these facilities
        since that is it's designated purpose as per:
        http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
--]]

local Armory = Armory;

ARMORY_MAX_LINE_TABS = 9;

ARMORYFRAME_MAINFRAMES = { "ArmoryFrame", "ArmoryLookupFrame", "ArmoryFindFrame" };
ARMORYFRAME_SUBFRAMES = { "ArmoryPaperDollFrame", "ArmoryPetFrame", "ArmoryTalentFrame", "ArmoryPVPFrame", "ArmoryOtherFrame" };
ARMORYFRAME_CHILDFRAMES = { "ArmoryTradeSkillFrame", "ArmoryInventoryFrame", "ArmoryQuestLogFrame", "ArmorySpellBookFrame", "ArmoryPVPTeamDetails", "ArmoryAchievementFrame", "ArmoryFriendsFrame" };

ARMORY_ID = "Armory";
ARMORYFRAME_SUBFRAME = "ArmoryPaperDollFrame";

StaticPopupDialogs["ARMORY_DELETE_CHARACTER"] = {
    text = ARMORY_DELETE_UNIT,
    button1 = YES,
    button2 = NO,
    OnAccept = function (self) ArmoryFrame_DeleteCharacter(self.data); end,
    OnHide = function (self) self.data = nil; end,
    timeout = 0,
    whileDead = 1,
    exclusive = 1,
    showAlert = 1,
    hideOnEscape = 1
};

StaticPopupDialogs["ARMORY_CHECK_MAIL_POPUP"] = {
    text = ARMORY_CHECK_MAIL_POPUP,
    button1 = OKAY,
    showAlert = 1,
    whileDead = 1,
    timeout = 0,
}

function ArmoryFrame_ToggleArmory(tab)
    local subFrame = _G[tab];
    if ( subFrame ) then
        PanelTemplates_SetTab(ArmoryFrame, subFrame:GetID());
        if ( ArmoryFrame:IsVisible() ) then
            if ( subFrame:IsVisible() ) then
                HideUIPanel(ArmoryFrame);
            else
                PlaySound("igCharacterInfoTab");
                ArmoryFrame_ShowSubFrame(tab);
            end
        else
            ShowUIPanel(ArmoryFrame);
            ArmoryFrame_ShowSubFrame(tab);
        end
    end
end

function ArmoryFrame_ShowSubFrame(frameName)
    for index, value in pairs(ARMORYFRAME_SUBFRAMES) do
        if ( value == frameName ) then
            _G[value]:Show();
            ARMORYFRAME_SUBFRAME = value;
        else
            _G[value]:Hide();
        end
    end
end

function ArmoryFrame_OnLoad(self)
    Armory:Init();

    -- Sliding frame
    --this:SetAttribute("UIPanelLayout-defined", true);
    --this:SetAttribute("UIPanelLayout-enabled", true);
    --this:SetAttribute("UIPanelLayout-area", "left");
    --this:SetAttribute("UIPanelLayout-pushable", 5);
    --this:SetAttribute("UIPanelLayout-whileDead", true);

    self:RegisterEvent("ADDON_LOADED");
    self:RegisterEvent("VARIABLES_LOADED");
    self:RegisterEvent("UNIT_NAME_UPDATE");
    self:RegisterEvent("PLAYER_PVP_RANK_CHANGED");
    self:RegisterEvent("PLAYER_UPDATE_RESTING");
    self:RegisterEvent("PLAYER_LOGOUT");
    self:RegisterEvent("TIME_PLAYED_MSG");
    self:RegisterEvent("PLAYER_ENTERING_WORLD");
    self:RegisterEvent("PLAYER_ENTER_COMBAT");
    self:RegisterEvent("PLAYER_LEAVE_COMBAT");
    self:RegisterEvent("PLAYER_REGEN_DISABLED");
    self:RegisterEvent("PLAYER_REGEN_ENABLED");

    -- Tab Handling code
    PanelTemplates_SetNumTabs(self, #ARMORYFRAME_SUBFRAMES);
    PanelTemplates_SetTab(self, 1);

    -- Allows Armory to be closed with the Escape key
    table.insert(UISpecialFrames, "ArmoryFrame");
end

function ArmoryFrame_OnEvent(self, event, ...)
    local arg1, arg2 = ...;

    if ( event == "ADDON_LOADED" ) then
        if ( arg1 == "Armory" ) then
            self.init = true;
            if ( IsAddOnLoaded("oGlow") ) then
                Armory_oGlow_OnLoad();
            end
        elseif ( arg1 == "EnhTooltip" ) then
            Armory_EnhTooltip_OnLoad();
        elseif ( arg1 == "ManyItemTooltips" ) then
            Armory_MIT_OnLoad();
        end
        if ( self.init ) then
            ArmoryMinimapButton_Init();
        end
    elseif ( event == "VARIABLES_LOADED" ) then
        Armory:InitDb();
        Armory:SetProfile(Armory:CurrentProfile());

        ArmoryChatInit();
        ArmoryTooltipsInit();
        Armory:PrepareMenu();
        Armory:LoadPawnScales();

        Armory:RemoveOldAuctions();
        local expire = Armory:CheckMailItems(true);
        if ( expire > 0 ) then
            StaticPopup_Show("ARMORY_CHECK_MAIL_POPUP", expire);
        end
        Armory:ExecuteDelayed(5, ArmoryFrame_RequestTimePlayed);
    elseif ( not Armory:CanHandleEvents() ) then
        return;
    elseif ( (event == "UNIT_NAME_UPDATE" and arg1 == "player") or event == "PLAYER_PVP_RANK_CHANGED" ) then
        Armory:Execute(ArmoryFrame_UpdateName);
    elseif ( event == "PLAYER_UPDATE_RESTING" ) then
        Armory:Execute(ArmoryFrame_UpdateResting);
    elseif ( event == "CHAT_MSG_ADDON" and arg1 == ARMORY_ID ) then
        ArmoryPaperDollFrame_UpdateVersion(arg2);
    elseif ( event == "PLAYER_LOGOUT" ) then
        Armory:DeleteInspectDb();
        Armory:UpdatePawnScales();
        Armory:SetTimePlayed(Armory:GetTimePlayed("player"));
    elseif ( event == "TIME_PLAYED_MSG" ) then
        Armory.hasTimePlayed = true;
        Armory:SetTimePlayed(arg1);
    elseif ( event == "PLAYER_ENTERING_WORLD" ) then
        Armory.inCombat = false;
        Armory.onHateList = false;
    elseif ( event == "PLAYER_ENTER_COMBAT" ) then
        Armory.inCombat = true;
    elseif ( event == "PLAYER_LEAVE_COMBAT" ) then
        Armory.inCombat = false;
    elseif ( event == "PLAYER_REGEN_DISABLED" ) then
        Armory.onHateList = true;
    elseif ( event == "PLAYER_REGEN_ENABLED" ) then
        Armory.onHateList = false;
    end

    if ( (Armory.inCombat or Armory.onHateList) and Armory:GetConfigPauseWhileInCombat() ) then
        Armory.commandHandler:Pause();
    else
        Armory.commandHandler:Resume();
    end
end

function ArmoryFrame_UpdateName()
    ArmoryNameText:SetText(Armory:UnitPVPName("player"));
end

function ArmoryFrame_UpdateResting()
    if ( Armory:IsResting() ) then
        ArmoryRestIcon:Show();
    else
        ArmoryRestIcon:Hide();
    end
end

local ChatFrame_DisplayTimePlayed_Orig = ChatFrame_DisplayTimePlayed;
function ChatFrame_DisplayTimePlayed(...)
    if ( not Armory.requestedTimePlayed ) then
        return ChatFrame_DisplayTimePlayed_Orig(...);
    end
    Armory.requestedTimePlayed = nil;
end

function ArmoryFrame_RequestTimePlayed()
    if ( not Armory.hasTimePlayed ) then
        Armory.requestedTimePlayed = true;
        RequestTimePlayed();
    end
end

function ArmoryFrame_OnShow(self)
    PlaySound("igCharacterInfoOpen");
    ArmoryFrame_Update(Armory:CurrentProfile());
end

function ArmoryFrame_OnHide(self)
    PlaySound("igCharacterInfoClose");
end

function ArmoryFrameTab_OnClick(self)
    local id = self:GetID();

    if ( id == 1 ) then
        ArmoryFrame_ToggleArmory("ArmoryPaperDollFrame");
    elseif ( id == 2 ) then
        ArmoryFrame_ToggleArmory("ArmoryPetFrame");
    elseif ( id == 3 ) then
        ArmoryFrame_ToggleArmory("ArmoryTalentFrame");
    elseif ( id == 4 ) then
        ArmoryFrame_ToggleArmory("ArmoryPVPFrame");
    elseif ( id == 5 ) then
        ArmoryFrame_ToggleArmory("ArmoryOtherFrame");
    end
    PlaySound("igCharacterInfoTab");
end

function ArmoryFrame_Update(profile, refresh)
    Armory:SetProfile(profile);
    Armory:SetPortraitTexture(ArmoryFramePortrait, "player");
    ArmoryFrame_UpdateName();
    ArmoryFrame_UpdateResting();
    ArmoryFrame_UpdateLineTabs();
    ArmoryAlternateSlotFrame_HideSlots();
    ArmoryPetTab_Update();

    if ( Armory:IsInspectCharacter() ) then
        PanelTemplates_SetTab(ArmoryFrame, 1);
        if ( not ArmoryPaperDollFrame:IsVisible() ) then
            ArmoryFrame_ToggleArmory("ArmoryPaperDollFrame");
        end
        ArmoryFrameTab4:Hide();
        ArmoryFrameTab5:Hide();
    else
        ArmoryFrameTab4:Show();
        ArmoryFrameTab5:Show();
    end

    if ( table.getn(Armory:Profiles()) > 1 ) then
        ArmoryNameDropDownButton:Enable();
        ArmoryFrameLeftButton:Enable();
        ArmoryFrameRightButton:Enable();
    else
        ArmoryNameDropDownButton:Disable();
        ArmoryFrameLeftButton:Disable();
        ArmoryFrameRightButton:Disable();
    end

    if ( refresh ) then
        local subFrameUpdate = _G[ARMORYFRAME_SUBFRAME.."_OnShow"];
        if ( subFrameUpdate ) then
            subFrameUpdate(_G[ARMORYFRAME_SUBFRAME]);
        end
        ArmoryCloseChildWindows(true);
    end
end

function ArmoryNameDropDown_OnLoad(self)
    ArmoryDropDownMenu_Initialize(self, ArmoryNameDropDown_Initialize);
    ArmoryDropDownMenu_SetWidth(self, 10);
end

function ArmoryNameDropDown_Initialize()
    -- Setup buttons
    local currentProfile = Armory:CurrentProfile();
    local unit = "player";
    local info, checked;

    if ( not Armory:ProfileExists(currentProfile) ) then
        currentProfile = {realm=Armory.playerRealm, character=Armory.player};
    end

    for _, realm in ipairs(Armory:RealmList()) do
        info = ArmoryDropDownMenu_CreateInfo();
        info.text = realm;
        info.notClickable = 1;
        info.notCheckable = 1;
        info.isTitle = 1;
        ArmoryDropDownMenu_AddButton(info);
        for _, character in ipairs(Armory:CharacterList(realm)) do
            local profile = {realm=realm, character=character};
            Armory:SelectProfile(profile);
            if ( Armory:UnitLevel(unit) and Armory:UnitRace(unit) and Armory:UnitClass(unit) ) then
                if ( realm == currentProfile.realm and character == currentProfile.character ) then
                    checked = 1;
                    ArmoryDropDownMenu_SetSelectedValue(ArmoryNameDropDown, profile);
                else
                    checked = nil;
                end

                info = ArmoryDropDownMenu_CreateInfo();
                info.text = character;
                info.func = ArmoryNameDropDown_OnClick;
                info.value = profile;
                info.checked = checked;
                info.tooltipTitle = Armory:UnitPVPName(unit);
                info.tooltipText = format(PLAYER_LEVEL, Armory:UnitLevel(unit), Armory:UnitRace(unit), Armory:UnitClass(unit));
                ArmoryDropDownMenu_AddButton(info);
            end
        end
    end
    Armory:SelectProfile(currentProfile);
end

function ArmoryNameDropDown_OnClick(self)
    ArmoryDropDownMenu_SetSelectedValue(ArmoryNameDropDown, self.value);
    ArmoryFrame_Update(self.value, true);
end

function ArmoryFrame_DeleteCharacter(data)
    local profile = Armory:CurrentProfile();
    if ( data.realm == profile.realm and data.character == profile.character ) then
        profile = ArmoryFrameCharacterCycle(false, true);
    end
    Armory:DeleteProfile(data.realm, data.character, true);
    ArmoryFrame_Update(profile, true);
end

function ArmoryFrame_UpdateLineTabs()
    local tabId = 1;

    if ( not Armory:IsInspectCharacter() ) then
        if ( Armory:HasInventory() ) then
            ArmoryFrame_SetLineTab(tabId, "Inventory", INVENTORY_TOOLTIP, "Interface\\Icons\\INV_Misc_Bag_08");
            tabId = tabId + 1;
        end

        if ( Armory:HasQuestLog() ) then
            ArmoryFrame_SetLineTab(tabId, "QuestLog", QUESTLOG_BUTTON, "Interface\\Icons\\INV_Misc_Book_08");
            tabId = tabId + 1;
        end

        if ( Armory:HasSpellBook() ) then
            ArmoryFrame_SetLineTab(tabId, "SpellBook", SPELLBOOK_BUTTON, "Interface\\Icons\\INV_Misc_Book_09");
            tabId = tabId + 1;
        end

        if ( Armory:HasAchievements() and Armory:GetTotalAchievementPoints() > 0 ) then
            ArmoryFrame_SetLineTab(tabId, "Achievements", ACHIEVEMENT_BUTTON, "Interface\\Icons\\Achievement_Level_10");
            tabId = tabId + 1;
        end

        if ( Armory:HasTradeSkills() ) then
            for _, name in ipairs(Armory:GetProfessionNames()) do
                local lineTab = ArmoryFrame_SetLineTab(tabId, "TradeSkill", name, Armory:GetProfessionTexture(name));
                if ( lineTab ) then
                    lineTab.skillName = name;
                    tabId = tabId + 1;
                end
            end
        end
    end

    -- Hide unused tabs
    for i = tabId, ARMORY_MAX_LINE_TABS do
        _G["ArmoryFrameLineTab"..i]:Hide();
    end
end

function ArmoryFrame_SetLineTab(id, tabType, tooltip, texture)
    if ( id and id > 0 and id <= ARMORY_MAX_LINE_TABS ) then
        local lineTab = _G["ArmoryFrameLineTab"..id];
        if ( lineTab ) then
            lineTab:SetNormalTexture(texture);
            lineTab.tooltip = tooltip;
            lineTab.tabType = tabType;
            lineTab:Show();
        end
        return lineTab;
    end
end

function ArmoryFrameLineTabTooltip(self)
    if ( self.tooltip ) then
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
        GameTooltip:SetText(self.tooltip);
    end
end

function ArmoryFrameLineTab_OnClick(self)
    for i = 1, ARMORY_MAX_LINE_TABS do
        local lineTab = _G["ArmoryFrameLineTab"..i];
        if ( lineTab:GetID() ~= self:GetID() ) then
            lineTab:SetChecked(nil);
        end
    end

    if ( self.tabType == "Inventory" ) then
        ArmoryInventoryFrame_Toggle();
    elseif ( self.tabType == "QuestLog" ) then
        ArmoryQuestLogFrame_Toggle();
    elseif ( self.tabType == "SpellBook" ) then
        ArmoryToggleSpellBook(BOOKTYPE_SPELL);
    elseif ( self.tabType == "Achievements" ) then
        ArmoryAchievementFrame_Toggle()
    elseif ( self.tabType == "TradeSkill" ) then
        if ( ArmoryTradeSkillFrame:IsShown() and self.skillName == Armory:GetSelectedProfession() ) then
            ArmoryTradeSkillFrame_Hide();
            return;
        end
        Armory:SetSelectedProfession(self.skillName);
        ArmoryTradeSkillFrame_Show();
    end
end

function ArmoryFrameLeft_Click(self)
    ArmoryFrameCharacterCycle(false);
end

function ArmoryFrameLeft_OnEnter(self)
    local profile = ArmoryFrameCharacterCycle(false, true);
    GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT");
    if ( profile ) then
        GameTooltip:SetText(profile.character, 1.0, 1.0, 1.0);
        GameTooltip:AddLine(profile.realm);
        GameTooltip:SetScale(0.75);
        GameTooltip:Show();
        self.UpdateTooltip = ArmoryFrameLeft_OnEnter;
    else
        self.UpdateTooltip = nil;
    end
end

function ArmoryFrameRight_Click(self)
    ArmoryFrameCharacterCycle(true);
end

function ArmoryFrameRight_OnEnter(self)
    local profile = ArmoryFrameCharacterCycle(true, true);
    GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT");
    if ( profile ) then
        GameTooltip:SetText(profile.character, 1.0, 1.0, 1.0);
        GameTooltip:AddLine(profile.realm);
        GameTooltip:SetScale(0.75);
        GameTooltip:Show();
        self.UpdateTooltip = ArmoryFrameRight_OnEnter;
    else
        self.UpdateTooltip = nil;
    end
end

function ArmoryFrameCharacterCycle(next, peek)
    local currentRealm, currentCharacter = Armory:GetPaperDollLastViewed();
    local profiles = Armory:Profiles();
    local selected = 0;

    for index, profile in ipairs(profiles) do
        if ( profile.realm == currentRealm and profile.character == currentCharacter ) then
            selected = index;
            break;
        end
    end

    if ( next ) then
        selected = selected + 1;
    else
        selected = selected - 1;
    end
    if ( selected > #profiles ) then
        selected = 1;
    elseif ( selected < 1 ) then
        selected = #profiles;
    end

    if ( peek ) then
        return profiles[selected];
    end

    ArmoryFrameSelectCharacter(profiles[selected])
end

function ArmoryFrameSelectCharacter(profile)
    ArmoryFrame_Update(profile, true);
    ArmoryCloseDropDownMenus();
    Armory_EQC_Refresh();
    if ( not ArmoryFrame:IsShown() ) then
        local text = profile.character;
        if ( table.getn(Armory:RealmList()) > 1 ) then
            if ( profile.realm == GetRealmName() ) then
                text = text..RED_FONT_COLOR_CODE;
            end
            text = text.." ("..profile.realm..")";
        end
        ArmoryMessageFrame:AddMessage(text);
        ArmoryMessageFrame:Show();
    end
end

function ArmoryCloseChildWindows(reopen)
    local childWindow, currentChild;
    for index, value in pairs(ARMORYFRAME_CHILDFRAMES) do
        childWindow = _G[value];
        if ( childWindow ) then
            if ( childWindow:IsVisible() ) then
                currentChild = childWindow;
            end
            childWindow:Hide();
        end
    end
    if ( reopen and currentChild ) then
        if ( currentChild:GetName() == "ArmoryTradeSkillFrame" ) then
            for _, name in ipairs(Armory:GetProfessionNames()) do
                if ( name == Armory:GetSelectedProfession() ) then
                     Armory:SetSelectedProfession(name);
                     ArmoryTradeSkillFrame_Show();
                     break;
                end
            end
        else
            currentChild:Show();
        end
    end
end

function ArmoryChildWindow_OnMouseUp(self, button)
    if ( ArmoryFrame.isMoving ) then
        ArmoryFrame:StopMovingOrSizing();
        ArmoryFrame.isMoving = false;
    end
end

function ArmoryChildWindow_OnMouseDown(self, button)
    if ( ( ( not ArmoryFrame.isLocked ) or ( ArmoryFrame.isLocked == 0 ) ) and ( button == "LeftButton" ) ) then
        ArmoryFrame:StartMoving();
        ArmoryFrame.isMoving = true;
    end
end

function ArmoryMinimapButton_Init()
    if ( Armory:GetConfigShowMinimap() ) then
        if ( Armory:GetConfigHideMinimapIfToolbar() and (IsAddOnLoaded("FuBar") or IsAddOnLoaded("Titan")) ) then
            ArmoryMinimapButton:Hide();
        else
            ArmoryMinimapButton_Move();
            ArmoryMinimapButton:Show();
        end
    else
        ArmoryMinimapButton:Hide();
    end
end

function ArmoryMinimapButton_OnLoad(self)
    self:RegisterForClicks("LeftButtonUp", "RightButtonUp");
    self:RegisterForDrag("LeftButton");
    self.updateDelay = 0;
end

function ArmoryMinimapButton_OnEnter(self)
    if ( not self.isMoving ) then
        Armory.LDB.OnEnter(self);
    end
end

function ArmoryMinimapButton_OnLeave(self)
    Armory.LDB.OnLeave();
end

function ArmoryMiniMapButton_OnClick(self, button)
    if ( not self.isMoving ) then
        Armory.LDB.OnClick(self, button);
    end
end

function ArmoryMinimapButton_OnUpdate(self, elapsed)
    self.updateDelay = self.updateDelay + elapsed;

    if ( self.isMoving ) then
        local xpos, ypos = GetCursorPosition();
        local xmin, ymin = Minimap:GetLeft(), Minimap:GetBottom();
        local angle;

        xpos = xmin - xpos / Minimap:GetEffectiveScale() + 70;
        ypos = ypos / Minimap:GetEffectiveScale() - ymin - 70;

        angle = math.deg(math.atan2(ypos, xpos));
        if ( angle < 0 ) then
            angle = angle + 360;
        end

        Armory:SetConfigMinimapAngle(angle);
        ArmoryOptionsMinimapPanelAngleSlider:SetValue(angle);

    elseif ( self.updateDelay > 0.5 ) then
        self.updateDelay = 0;

        if ( Armory.dbLoaded ) then
            ArmoryMinimapButtonIcon:SetTexture(Armory:GetPortraitTexture("player"));
        end
    end
end

function ArmoryMinimapButton_Move()
    local angle = Armory:GetConfigMinimapAngle();
    local radius = Armory:GetConfigMinimapRadius();
    local xpos = radius * cos(angle);
    local ypos = radius * sin(angle);

    ArmoryMinimapButton:SetPoint("TOPLEFT", "Minimap", "TOPLEFT", 54 - xpos, ypos - 55);
end

local Orig_GameTooltip_ShowCompareItem = GameTooltip_ShowCompareItem;
function GameTooltip_ShowCompareItem(...)
    if ( ArmoryComparisonTooltip1:IsVisible() or ArmoryComparisonTooltip2:IsVisible() ) then
        return;
    end
    return Orig_GameTooltip_ShowCompareItem(...);
end

function ArmoryComparisonFrame_OnUpdate(self, elapsed)
    local link, tooltip;

    self.updateTime = self.updateTime - elapsed;
    if ( self.updateTime > 0 ) then
        return;
    end
    self.updateTime = TOOLTIP_UPDATE_TIME;

    if ( not Armory:GetConfigShowEqcTooltips() ) then
        return;
    elseif ( GameTooltip:IsVisible() ) then
        tooltip = GameTooltip;
    elseif ( ItemRefTooltip:IsVisible() ) then
        tooltip = ItemRefTooltip;
    end
    self.tooltip = tooltip;

    if ( IsAltKeyDown() and tooltip ) then
        _, link = tooltip:GetItem();
        if ( self.link ~= link ) then
            self.link = link;
            self.hasShoppingTooltips = ShoppingTooltip1:IsVisible() or ShoppingTooltip2:IsVisible() or ShoppingTooltip3:IsVisible();
            if ( link ) then
                self.hasComparison = true;

                ShoppingTooltip1:Hide();
                ShoppingTooltip2:Hide();
                ShoppingTooltip3:Hide();

                ArmoryShowCompareItem(tooltip, link);
            end
        end

    elseif ( self.hasComparison ) then
        self.hasComparison = false;
        self.link = nil;

        ArmoryComparisonTooltip1:Hide();
        ArmoryComparisonTooltip2:Hide();

        if ( GameTooltip:IsVisible() and self.hasShoppingTooltips ) then
            GameTooltip_ShowCompareItem();
        end

    end
end

local compareSlots = {};
function ArmoryShowCompareItem(tooltip, link)
    ArmoryComparisonTooltip1:Hide();
    ArmoryComparisonTooltip2:Hide();

    if ( not link or link == "" ) then
        return;
    end

    local _, _, _, _, _, _, _, _, equipLoc = GetItemInfo(link);
    local slot = ARMORY_SLOTINFO[equipLoc];
    if ( not slot ) then
        return;
    elseif ( slot:match("Finger.Slot") ) then
        compareSlots[1] = "Finger0Slot";
        compareSlots[2] = "Finger1Slot";
    elseif ( slot:match("Trinket.Slot") ) then
        compareSlots[1] = "Trinket0Slot";
        compareSlots[2] = "Trinket1Slot";
    elseif ( slot == "MainHandSlot" or slot == "SecondaryHandSlot" ) then
        compareSlots[1] = "SecondaryHandSlot";
        compareSlots[2] = "MainHandSlot";
    else
        compareSlots[1] = slot;
        compareSlots[2] = nil;
    end

    local slotId = GetInventorySlotInfo(compareSlots[1]);
    local itemLink = Armory:GetInventoryItemLink("player", slotId);
    if ( not itemLink ) then
        compareSlots[1] = compareSlots[2];
        compareSlots[2] = nil;
    end
    if ( not compareSlots[1] ) then
        return;
    end

    -- find correct side
    local side = "left";
    local rightDist = 0;
    local leftPos = tooltip:GetLeft();
    local rightPos = tooltip:GetRight();
    if ( not rightPos ) then
        rightPos = 0;
    end
    if ( not leftPos ) then
        leftPos = 0;
    end

    rightDist = GetScreenWidth() - rightPos;

    if (leftPos and (rightDist < leftPos)) then
        side = "left";
    else
        side = "right";
    end

    -- see if we should slide the tooltip
    if ( tooltip:GetAnchorType() ) then
        local totalWidth = 0;
        if ( compareSlots[1]  ) then
            Armory:SetInventoryItem("player", GetInventorySlotInfo(compareSlots[1]), nil, ArmoryComparisonTooltip1);
            totalWidth = totalWidth + ArmoryComparisonTooltip1:GetWidth();
        end
        if ( compareSlots[2]  ) then
            Armory:SetInventoryItem("player", GetInventorySlotInfo(compareSlots[2]), nil, ArmoryComparisonTooltip2);
            totalWidth = totalWidth + ArmoryComparisonTooltip2:GetWidth();
        end

        if ( (side == "left") and (totalWidth > leftPos) ) then
            tooltip:SetAnchorType(tooltip:GetAnchorType(), (totalWidth - leftPos), 0);
        elseif ( (side == "right") and (rightPos + totalWidth) >  GetScreenWidth() ) then
            tooltip:SetAnchorType(tooltip:GetAnchorType(), -((rightPos + totalWidth) - GetScreenWidth()), 0);
        end
    end

    -- anchor the compare tooltips
    if ( compareSlots[1] ) then
        ArmoryComparisonTooltip1:SetOwner(tooltip, "ANCHOR_NONE");
        ArmoryComparisonTooltip1:ClearAllPoints();
        if ( side and side == "left" ) then
            ArmoryComparisonTooltip1:SetPoint("TOPRIGHT", tooltip:GetName(), "TOPLEFT", 0, -10);
        else
            ArmoryComparisonTooltip1:SetPoint("TOPLEFT", tooltip:GetName(), "TOPRIGHT", 0, -10);
        end
        Armory:SetInventoryItem("player", GetInventorySlotInfo(compareSlots[1]), nil, ArmoryComparisonTooltip1);

        if ( compareSlots[2] ) then
            ArmoryComparisonTooltip2:SetOwner(ArmoryComparisonTooltip1, "ANCHOR_NONE");
            ArmoryComparisonTooltip2:ClearAllPoints();
            if ( side and side == "left" ) then
                ArmoryComparisonTooltip2:SetPoint("TOPRIGHT", "ArmoryComparisonTooltip1", "TOPLEFT", 0, 0);
            else
                ArmoryComparisonTooltip2:SetPoint("TOPLEFT", "ArmoryComparisonTooltip1", "TOPRIGHT", 0, 0);
            end
            Armory:SetInventoryItem("player", GetInventorySlotInfo(compareSlots[2]), nil, ArmoryComparisonTooltip2);
        end
    end
end

function Armory_EQC_Refresh()
    local frame = ArmoryComparisonFrame;
    if ( frame.hasComparison ) then
        ArmoryShowCompareItem(frame.tooltip, frame.link);
    end

    if ( EquipCompare_PostClearTooltip ) then
        EquipCompare_PostClearTooltip();
    end
end

function ArmoryTooltipsInit()
    local gttSetItemFunc = GameTooltip:GetScript("OnTooltipSetItem");
    local gttSetSpellFunc = GameTooltip:GetScript("OnTooltipSetSpell");
    local gttClearedFunc = GameTooltip:GetScript("OnTooltipCleared");

    GameTooltip:SetScript("OnTooltipSetItem", function(self, ...) return ArmoryEnhanceTooltip(gttSetItemFunc, self, ...) end);
    GameTooltip:SetScript("OnTooltipSetSpell", function(self, ...) return ArmoryEnhanceSpellTooltip(gttSetSpellFunc, self, ...) end);
    GameTooltip:SetScript("OnTooltipCleared", function(self, ...) return ArmoryClearTooltip(gttClearedFunc, self, ...) end);

    local irtSetItemFunc = ItemRefTooltip:GetScript("OnTooltipSetItem");
    local irtSetSpellFunc = ItemRefTooltip:GetScript("OnTooltipSetSpell");
    local irtClearedFunc = ItemRefTooltip:GetScript("OnTooltipCleared");

    ItemRefTooltip:SetScript("OnTooltipSetItem", function(self, ...) return ArmoryEnhanceTooltip(irtSetItemFunc, self, ...) end);
    ItemRefTooltip:SetScript("OnTooltipSetSpell", function(self, ...) return ArmoryEnhanceSpellTooltip(irtSetSpellFunc, self, ...) end);
    ItemRefTooltip:SetScript("OnTooltipCleared", function(self, ...) return ArmoryClearTooltip(irtClearedFunc, self, ...) end);
end

function ArmoryEnhanceTooltip(origFunc, tooltip, ...)
    if ( origFunc ) then
        origFunc(tooltip, ...);
    end

    if ( not tooltip or tooltip:GetName() == "ArmoryTooltip" ) then
        return;
    elseif ( not tooltip.hasArmoryCounts ) then
        local name, link = tooltip:GetItem();

        if ( link ~= Armory.tooltipItem ) then
            Armory.tooltipItem = link;
            Armory.tooltipRecipeCounts = nil;
            Armory.tooltipItemCounts = nil;
            if ( link ) then
                Armory.tooltipRecipeCounts = Armory:GetRecipeOwners(name, link);
                Armory.tooltipItemCounts = Armory:GetItemCount(link);
            end
        end

        if ( Armory.tooltipRecipeCounts and Armory.tooltipItemCounts ) then
            tooltip.hasArmoryCounts = true;

            local r, g, b = Armory:GetConfigRecipeKnownColor();
            for k, v in ipairs(Armory.tooltipRecipeCounts) do
                tooltip:AddDoubleLine(USED, v.name, r, g, b, r, g, b);
            end

            local count, bagCount, bankCount, mailCount, auctionCount = 0, 0, 0, 0, 0;
            local details;
            r, g, b = Armory:GetConfigItemCountColor();
            for k, v in ipairs(Armory.tooltipItemCounts) do
                count = count + v.count;
                bagCount = bagCount + (v.bags or 0);
                bankCount = bankCount + (v.bank or 0);
                mailCount = mailCount + (v.mail or 0);
                auctionCount = auctionCount + (v.auction or 0);
                details = v.details or Armory:GetCountDetails(v.bags, v.bank, v.mail, v.auction);
                tooltip:AddDoubleLine(format("%s [%d]", v.name, v.count), details, r, g, b, r, g, b);
            end

            if ( count > 0 and Armory:GetConfigShowItemCountTotals() ) then
                r, g, b = Armory:GetConfigItemCountTotalsColor();
                local guildCount = count - bagCount - bankCount - mailCount - auctionCount;
                details = Armory:GetCountDetails(bagCount, bankCount, mailCount, auctionCount, nil, guildCount);
                tooltip:AddDoubleLine(format(ARMORY_TOTAL, count), details, r, g, b, r, g, b);
            end
            tooltip:Show();
        end
    end
end

function ArmoryEnhanceSpellTooltip(origFunc, tooltip, ...)
    if ( origFunc ) then
        origFunc(tooltip, ...);
    end

    if ( not tooltip or tooltip:GetName() == "ArmoryTooltip" ) then
        return;
    elseif ( not (Armory:HasInventory() and Armory:GetConfigShowItemCount()) ) then
        return;
    elseif ( not tooltip.hasArmoryCounts ) then
        local name = tooltip:GetSpell();

        if ( name ~= Armory.tooltipItem ) then
            Armory.tooltipItem = name;
            Armory.tooltipReagents = nil;
            Armory.tooltipItemCounts = nil;
            
            if ( name ) then
                local fontString, text, quantity;
                for i = 1, tooltip:NumLines() do
                    fontString = _G[tooltip:GetName().."TextLeft"..i];
                    if ( fontString and fontString:IsShown() ) then
                        text = fontString:GetText();
                        if ( text:match(SPELL_REAGENTS.."(.*)") ) then
                            Armory.tooltipReagents = Armory:StringSplit(",", text:match(SPELL_REAGENTS.."(.*)"));
                            for k, v in ipairs(Armory.tooltipReagents) do
                                v = strtrim(v);
                                if ( v:match("|c%x%x%x%x%x%x%x%x(.-)|r") ) then
                                    Armory.tooltipReagents[k] = v:match("|c%x%x%x%x%x%x%x%x(.-)|r");
                                else
                                    Armory.tooltipReagents[k] = v;
                                end
                                if ( Armory.tooltipReagents[k]:match("%d%)$") ) then
                                    quantity = tonumber(Armory.tooltipReagents[k]:match("%((%d+)%)$"));
                                    text = strtrim(Armory.tooltipReagents[k]:gsub("%(%d+%)$", ""));
                                else
                                    quantity = 1;
                                    text = Armory.tooltipReagents[k];
                                end
                                Armory.tooltipReagents[k] = {text, quantity};
                            end
                            break;
                        end
                    end
                end
                
                if ( Armory.tooltipReagents ) then
                    Armory.tooltipItemCounts = Armory:GetMultipleItemCount(Armory.tooltipReagents);
                end
            end
        end

        if ( Armory.tooltipReagents and Armory.tooltipItemCounts ) then
            tooltip.hasArmoryCounts = true;

            local reagents = Armory.tooltipReagents;
            local itemCounts = Armory.tooltipItemCounts;
            local count, bags, bank, mail, auction, alts;
            local name, quantity, details;
            local r, g, b = Armory:GetConfigItemCountColor();
            
            if ( #itemCounts > 0 ) then
                for i = 1, #reagents do
                    name, quantity = unpack(reagents[i]);
                    count, bags, bank, mail, auction, alts = 0, 0, 0, 0, 0, 0;
                    for _, v in ipairs(itemCounts[i]) do
                        if ( v.mine ) then
                            bags = bags + (v.bags or 0);
                            bank = bank + (v.bank or 0);
                            mail = mail + (v.mail or 0);
                            auction = auction + (v.auction or 0);
                        else
                            alts = alts + (v.bags or 0) + (v.bank or 0) + (v.mail or 0) + (v.auction or 0);
                        end
                        count = count + v.count;
                    end
                    details = Armory:GetCountDetails(bags, bank, mail, auction, alts, count - bags - bank - mail - auction - alts);
                    tooltip:AddDoubleLine(name..format(" [%d/%d]", count, quantity), details, r, g, b, r, g, b);
                end
                tooltip:Show();
            end
        end
    end
end

function ArmoryClearTooltip(origFunc, tooltip, ...)
    if ( origFunc ) then
        origFunc(tooltip, ...);
    end
    tooltip.hasArmoryCounts = nil;
end

----------------------------------------------------------
-- EnhTooltip support
----------------------------------------------------------

function Armory_EnhTooltip_OnLoad()
    Stubby.RegisterFunctionHook("Armory.SetQuestLogItem", 200, Armory_EnhTooltip_HookSetQuestLogItem);
    Stubby.RegisterFunctionHook("Armory.SetInventoryItem", 200, Armory_EnhTooltip_HookSetInventoryItem);
    Stubby.RegisterFunctionHook("Armory.SetBagItem", 200, Armory_EnhTooltip_HookSetBagItem);
    Stubby.RegisterFunctionHook("Armory.SetTradeSkillItem", 200, Armory_EnhTooltip_HookSetTradeSkillItem);
end

function Armory_EnhTooltip_HookSetQuestLogItem(funcArgs, retVal, frame, qtype, slot)
    local link = Armory:GetQuestLogItemLink(qtype, slot);
    if ( link ) then
        local name, texture, quantity, quality, usable = Armory:GetQuestLogRewardInfo(slot);
        name = name or Armory:GetNameFromLink(link);
        quality = Armory:GetQualityFromLink(link);
        return EnhTooltip.TooltipCall(GameTooltip, name, link, quality, quantity);
    end
end

function Armory_EnhTooltip_HookSetInventoryItem(funcArgs, retVal, frame, unit, slot, dontShow, tooltip, link)
    if ( (link or Armory:SetInventoryItemInfo(unit, slot)) and not dontShow and not tooltip ) then
        link = link or Armory:GetInventoryItemLink(unit, slot);
        if ( link ) then
            local name = Armory:GetNameFromLink(link);
            local quantity = 1;
            local quality = Armory:GetQualityFromLink(link);
            return EnhTooltip.TooltipCall(GameTooltip, name, link, quality, quantity);
        end
    end
end

function Armory_EnhTooltip_HookSetBagItem(funcArgs, retVal, frame, frameID, buttonID)
    local link = Armory:GetContainerItemLink(frameID, buttonID);
    local name = Armory:GetNameFromLink(link);
    if ( name ) then
        local texture, itemCount, locked, quality, readable = Armory:GetContainerItemInfo(frameID, buttonID);
        if ( not (quality and quality ~= -1) ) then
            quality = Armory:GetQualityFromLink(link);
        end
        return EnhTooltip.TooltipCall(GameTooltip, name, link, quality, itemCount);
    end
end

function Armory_EnhTooltip_HookSetTradeSkillItem(funcArgs, retVal, frame, skill, slot)
    local link;
    if ( slot ) then
        link = Armory:GetTradeSkillReagentItemLink(skill, slot);
        if ( link ) then
            local name, texture, quantity, quantityHave = Armory:GetTradeSkillReagentInfo(skill, slot);
            local quality = Armory:GetQualityFromLink(link);
            return EnhTooltip.TooltipCall(GameTooltip, name, link, quality, quantity);
        end
    else
        link = Armory:GetTradeSkillItemLink(skill);
        if ( link ) then
            local name = Armory:GetNameFromLink(link);
            local quality = Armory:GetQualityFromLink(link);
            return EnhTooltip.TooltipCall(GameTooltip, name, link, quality);
        end
    end
end


----------------------------------------------------------
-- oGlow support
----------------------------------------------------------

function Armory_oGlow_OnLoad()
    hooksecurefunc("ArmoryPaperDollItemSlotButton_Update", Armory_oGlow_PaperDollItemSlotButton_Update);
    hooksecurefunc(Armory, "SetItemLink", Armory_oGlow_SetItemLink);
end

local alertSlots = {
    "ArmoryHeadSlot",
    "ArmoryShoulderSlot",
    "ArmoryChestSlot",
    "ArmoryWaistSlot",
    "ArmoryLegsSlot",
    "ArmoryFeetSlot",
    "ArmoryWristSlot",
    "ArmoryHandsSlot",
    "ArmoryMainHandSlot",
    "ArmorySecondaryHandSlot",
    "ArmoryRangedSlot"
};
function Armory_oGlow_PaperDollItemSlotButton_Update(button)
    local getAlertStatus = function(buttonName)
        for index, value in pairs(alertSlots) do
            if ( value == buttonName ) then
                return Armory:GetInventoryAlertStatus(index) or 2;
            end
        end
        return 2;
    end
    
    local slotId = button:GetID();
    local quality = -1;
    local status = 2;
    local isBroken;
    
    if ( button.itemId ~= nil ) then
        if ( button.itemId ~= 0 ) then
            _, _, quality = _G.GetItemInfo(button.itemId);
        end
    else
        quality = Armory:GetInventoryItemQuality("player", slotId) or Armory:GetQualityFromLink(button.link);
        isBroken = Armory:GetInventoryItemBroken("player", slotId);
        status = getAlertStatus(button:GetName());

        if ( isBroken ) then
            quality = 100;
        elseif ( status == 3 ) then
            quality = 99;
        end
    end

    oGlow(button, quality);
end

function Armory_oGlow_SetItemLink(self, button, link)
    if ( link ) then
        local _, _, quality = GetItemInfo(link);
        local icon = _G[button:GetName().."IconTexture"];
        oGlow(button, quality or Armory:GetQualityFromLink(link), icon);
    elseif ( button.bc ) then
        button.bc:Hide();
    end
end

----------------------------------------------------------
-- ManyItemTooltips (MIT) support
----------------------------------------------------------

function Armory_MIT_OnLoad()
    MIT:AddHook("Armory", "OnShow", function(tooltip)
        if ( tooltip:GetSpell() ) then
            ArmoryEnhanceSpellTooltip(nil, tooltip);
        else
            ArmoryEnhanceTooltip(nil, tooltip);
        end
    end);
    MIT:AddHook("Armory", "OnHide", function(tooltip) 
        ArmoryClearTooltip(nil, tooltip);
    end);
end
