--[[
    Armory Addon for World of Warcraft(tm).
    Revision: 95 2009-10-02T18:28:46Z
    URL: http://www.wow-neighbours.com

    License:
        This program is free software; you can redistribute it and/or
        modify it under the terms of the GNU General Public License
        as published by the Free Software Foundation; either version 2
        of the License, or (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program(see GPL.txt); if not, write to the Free Software
        Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    Note:
        This AddOn's source code is specifically designed to work with
        World of Warcraft's interpreted AddOn system.
        You have an implicit licence to use this AddOn with these facilities
        since that is it's designated purpose as per:
        http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
--]]

local Armory = Armory;

ARMORY_FRIENDS_TO_DISPLAY = 11;
ARMORY_FRIENDS_FRAME_FRIEND_HEIGHT = 34;
ARMORY_IGNORES_TO_DISPLAY = 21;
ARMORY_FRIENDS_FRAME_IGNORE_HEIGHT = 16;

ARMORY_FRIENDSFRAME_SUBFRAMES = { "ArmoryFriendsListFrame", "ArmoryIgnoreListFrame" };
function ArmoryFriendsFrame_ShowSubFrame(frameName)
    for index, value in pairs(ARMORY_FRIENDSFRAME_SUBFRAMES) do
        if ( value == frameName ) then
            _G[value]:Show()
        else
            _G[value]:Hide();
        end	
    end 
end

function ArmoryFriendsFrame_Toggle()
    if ( ArmoryFriendsFrame:IsShown() or not Armory:HasFriends() ) then
        HideUIPanel(ArmoryFriendsFrame);
    else
        ArmoryCloseChildWindows();
        ShowUIPanel(ArmoryFriendsFrame);
    end
end

function ArmoryFriendsFrame_OnLoad(self)
    self:RegisterEvent("PLAYER_ENTERING_WORLD");
    self:RegisterEvent("FRIENDLIST_UPDATE");
    self:RegisterEvent("IGNORELIST_UPDATE");
    self.showFriendsList = 1;
end

function ArmoryFriendsFrame_OnEvent(self, event, ...)
    if ( not Armory:CanHandleEvents() ) then
        return;
    elseif ( event == "PLAYER_ENTERING_WORLD" ) then
        self:UnregisterEvent("PLAYER_ENTERING_WORLD");
    end
    Armory:Execute(ArmoryFriendsFrame_UpdateFriends);
end

function ArmoryFriendsFrame_UpdateFriends()
    Armory:UpdateFriends();
    if ( ArmoryFriendsListFrame:IsShown() ) then
        ArmoryFriendsList_Update();
    elseif ( ArmoryIgnoreListFrame:IsShown() ) then
        ArmoryIgnoreList_Update();
    end
end

function ArmoryFriendsFrame_Update()
    if ( ArmoryFriendsFrame.showFriendsList ) then
        ArmoryFriendsFrameTitleText:SetText(FRIENDS_LIST);
        ArmoryFriendsFrame_ShowSubFrame("ArmoryFriendsListFrame");
    else
        ArmoryFriendsFrameTitleText:SetText(IGNORE_LIST);
        ArmoryFriendsFrame_ShowSubFrame("ArmoryIgnoreListFrame");
        ArmoryIgnoreList_Update();
    end
end

function ArmoryFriendsFrame_OnShow(self)
	ArmoryFriendsList_Update();
	ArmoryFriendsFrame_Update();
end

function ArmoryFriendsFrame_OnHide(self)
	for index, value in pairs(ARMORY_FRIENDSFRAME_SUBFRAMES) do
		_G[value]:Hide();
	end
end

function ArmoryFriendsList_Update()
    local numFriends = Armory:GetNumFriends();
    local nameText, infoText, noteText, noteHiddenText;
    local name, class, note;
    local friendButton, noteFrame;

    local friendOffset = FauxScrollFrame_GetOffset(ArmoryFriendsListScrollFrame);
    local friendIndex;

    for i = 1, ARMORY_FRIENDS_TO_DISPLAY, 1 do
        friendIndex = friendOffset + i;
        name, class, note = Armory:GetFriendInfo(friendIndex);
        nameText = _G["ArmoryFriendsFrameFriendButton"..i.."ButtonTextName"];
        infoText = _G["ArmoryFriendsFrameFriendButton"..i.."ButtonTextInfo"];
        noteFrame = _G["ArmoryFriendsFrameFriendButton"..i.."ButtonTextNote"];
        noteText = _G["ArmoryFriendsFrameFriendButton"..i.."ButtonTextNoteText"];
        noteHiddenText = _G["ArmoryFriendsFrameFriendButton"..i.."ButtonTextNoteHiddenText"];
        friendButton = _G["ArmoryFriendsFrameFriendButton"..i];
        nameText:ClearAllPoints();
        nameText:SetPoint("TOPLEFT", 10, -3);
        noteFrame:SetPoint("RIGHT", nameText, "LEFT", 0, 0);
        friendButton:SetID(friendIndex);

        friendButton.candidate = nil;
        if ( not name ) then
            name = UNKNOWN;
        elseif ( Armory.characterRealm == Armory.playerRealm and Armory.player ~= Armory.character and Armory:UnitFactionGroup() == _G.UnitFactionGroup("player") ) then
            friendButton.candidate = name;
        end
        nameText:SetText(name);
        infoText:SetText(class);

        if ( note ) then
            noteText:SetFormattedText(FRIENDS_LIST_NOTE_TEMPLATE, note);
            noteHiddenText:SetText(note);
            local width = noteHiddenText:GetWidth() + infoText:GetWidth();
            local friendButtonWidth = friendButton:GetWidth();
            if ( ArmoryFriendsListScrollFrameScrollBarTop:IsVisible() ) then
                friendButtonWidth = friendButtonWidth - ArmoryFriendsListScrollFrameScrollBarTop:GetWidth();
            end
            if ( width > friendButtonWidth ) then
                width = friendButtonWidth - infoText:GetWidth();
            end
            noteText:SetWidth(width);
            noteText:SetHeight(14);
        else
            noteText:SetText("");
        end

        if ( friendIndex > numFriends ) then
            friendButton:Hide();
        else
            friendButton:Show();
        end
    end

    -- ScrollFrame stuff
    FauxScrollFrame_Update(ArmoryFriendsListScrollFrame, numFriends, ARMORY_FRIENDS_TO_DISPLAY, ARMORY_FRIENDS_FRAME_FRIEND_HEIGHT);
end

function ArmoryIgnoreList_Update()
    local numIgnores = Armory:GetNumIgnores();
    local nameText;
    local name;
    local ignoreButton;

    local ignoreOffset = FauxScrollFrame_GetOffset(ArmoryIgnoreListScrollFrame);
    local ignoreIndex;
    for i = 1, ARMORY_IGNORES_TO_DISPLAY do
        ignoreIndex = i + ignoreOffset;
        name = Armory:GetIgnoreName(ignoreIndex);
        nameText = _G["ArmoryFriendsFrameIgnoreButton"..i.."ButtonTextName"];
        ignoreButton = _G["ArmoryFriendsFrameIgnoreButton"..i];
        ignoreButton:SetID(ignoreIndex);

        ignoreButton.candidate = nil;
        if ( name ~= "" and Armory.characterRealm == Armory.playerRealm and Armory.player ~= Armory.character ) then
            ignoreButton.candidate = name;
        end
        nameText:SetText(name);

        if ( ignoreIndex > numIgnores ) then
            ignoreButton:Hide();
        else
            ignoreButton:Show();
        end
    end

    -- ScrollFrame stuff
    FauxScrollFrame_Update(ArmoryIgnoreListScrollFrame, numIgnores, ARMORY_IGNORES_TO_DISPLAY, ARMORY_FRIENDS_FRAME_IGNORE_HEIGHT);
end